@@ -7,6 +7,7 @@ from typing import List, Dict, Union
 
 # noinspection PyPackageRequirements
 from Crypto.PublicKey import RSA
+# noinspection PyPackageRequirements
 from Crypto.PublicKey.RSA import RsaKey
 from iteration_utilities import unique_everseen
 
@@ -14,6 +15,7 @@ from federation.entities.activitypub.constants import NAMESPACE_PUBLIC
 from federation.entities.mixins import BaseEntity
 from federation.protocols.activitypub.signing import get_http_authentication
 from federation.types import UserType
+from federation.utils.matrix import get_matrix_configuration
 from federation.utils.network import send_document
 
 logger = logging.getLogger("federation")
@@ -26,7 +28,7 @@ def handle_create_payload(
         to_user_key: RsaKey = None,
         parent_user: UserType = None,
         payload_logger: callable = None,
-) -> Union[str, dict]:
+) -> Union[str, Dict, List[Dict]]:
     """Create a payload with the given protocol.
 
     Any given user arguments must have ``private_key`` and ``handle`` attributes.
@@ -40,11 +42,13 @@ def handle_create_payload(
                       be generated. If given, the payload will be sent as this user.
     :arg payload_logger: (Optional) Function to log the payloads with.
 
-    :returns: Built payload (str or dict)
+    :returns: Built payload(s) (str or dict or list (of payloads))
     """
     mappers = importlib.import_module(f"federation.entities.{protocol_name}.mappers")
     protocol = importlib.import_module(f"federation.protocols.{protocol_name}.protocol")
+    # noinspection PyUnresolvedReferences
     protocol = protocol.Protocol()
+    # noinspection PyUnresolvedReferences
     outbound_entity = mappers.get_outbound_entity(entity, author_user.rsa_private_key)
     if parent_user:
         outbound_entity.sign_with_parent(parent_user.rsa_private_key)
@@ -119,6 +123,12 @@ def handle_send(
                             "protocol": "activitypub",
                             "public": False,
                         },
+                        {
+                            "endpoint": "https://matrix.domain.tld",
+                            "fid": "#@user:domain.tld",
+                            "protocol": "matrix",
+                            "public": True,
+                        }
                      ]
     :arg parent_user: (Optional) User object of the parent object, if there is one. This must be given for the
                       Diaspora protocol if a parent object exists, so that a proper ``parent_author_signature`` can
@@ -129,11 +139,19 @@ def handle_send(
     ready_payloads = {
         "activitypub": {
             "auth": None,
+            "headers": {},
             "payload": None,
             "urls": set(),
         },
         "diaspora": {
             "auth": None,
+            "headers": {},
+            "payload": None,
+            "urls": set(),
+        },
+        "matrix": {
+            "auth": None,
+            "headers": {},
             "payload": None,
             "urls": set(),
         },
@@ -141,11 +159,16 @@ def handle_send(
     skip_ready_payload = {
         "activitypub": False,
         "diaspora": False,
+        "matrix": False,
     }
-
+    logger.debug('handle_send - length of recipients: %s', len(recipients))
     # Flatten to unique recipients
     # TODO supply a callable that empties "fid" in the case that public=True
-    unique_recipients = unique_everseen(recipients)
+    unique_recipients = list(unique_everseen(recipients))
+    logger.debug('handle_send - length of unique_recipients: %s', len(unique_recipients))
+    logger.debug('handle_send / unique_recipients - %s', unique_recipients)
+
+    matrix_config = None
 
     # Generate payloads and collect urls
     for recipient in unique_recipients:
@@ -160,11 +183,14 @@ def handle_send(
 
         if protocol == "activitypub":
             if skip_ready_payload["activitypub"]:
+                logger.debug('Skipping activitypub payload as skip_ready_payload set')
                 continue
-            if entity.__class__.__name__.startswith("Diaspora"):
-                # Don't try to do anything with Diaspora entities currently
+            if entity.__class__.__name__.startswith("Diaspora") or entity.__class__.__name__.startswith("Matrix"):
+                # Don't try to do anything with these entities currently
                 skip_ready_payload["activitypub"] = True
+                logger.debug('Skipping activitypub payload as payload is diaspora or matrix')
                 continue
+            # noinspection PyBroadException
             try:
                 if not ready_payloads[protocol]["payload"]:
                     try:
@@ -191,7 +217,8 @@ def handle_send(
                 rendered_payload = json.dumps(payload).encode("utf-8")
             except Exception:
                 logger.error(
-                    "handle_send - failed to generate payload for %s, %s: %s", fid, endpoint, traceback.format_exc(),
+                    "handle_send - failed to generate activitypub payload for %s, %s: %s",
+                    fid, endpoint, traceback.format_exc(),
                     extra={
                         "recipient": recipient,
                         "unique_recipients": list(unique_recipients),
@@ -200,23 +227,27 @@ def handle_send(
                         "ready_payloads": ready_payloads,
                         "entity": entity,
                         "author": author_user.id,
-                        "parent_user": parent_user.id,
+                        "parent_user": parent_user.id if parent_user else None,
                     }
                 )
                 continue
             payloads.append({
                 "auth": get_http_authentication(author_user.rsa_private_key, f"{author_user.id}#main-key"),
+                "headers": {
+                    "Content-Type": 'application/ld+json; profile="https://www.w3.org/ns/activitystreams"',
+                },
                 "payload": rendered_payload,
-                "content_type": 'application/ld+json; profile="https://www.w3.org/ns/activitystreams"',
                 "urls": {endpoint},
             })
         elif protocol == "diaspora":
-            if entity.__class__.__name__.startswith("Activitypub"):
-                # Don't try to do anything with Activitypub entities currently
+            if entity.__class__.__name__.startswith("Activitypub") or entity.__class__.__name__.startswith("Matrix"):
+                # Don't try to do anything with these entities currently
                 skip_ready_payload["diaspora"] = True
+                logger.debug('Skipping diaspora payload as payload is activitypub or matrix')
                 continue
             if public:
                 if skip_ready_payload["diaspora"]:
+                    logger.debug('Skipping diaspora payload as skip_ready_payload set')
                     continue
                 if public_key:
                     logger.warning("handle_send - Diaspora recipient cannot be public and use encrypted delivery")
@@ -249,14 +280,76 @@ def handle_send(
                     logger.error("handle_send - failed to generate private payload for %s: %s", endpoint, ex)
                     continue
                 payloads.append({
-                    "urls": {endpoint}, "payload": payload, "content_type": "application/json", "auth": None,
+                    "auth": None,
+                    "headers": {
+                        "Content-Type": "application/json",
+                    },
+                    "payload": payload,
+                    "urls": {endpoint},
                 })
+        elif protocol == "matrix":
+            if skip_ready_payload["matrix"]:
+                logger.debug('Skipping matrix payload as skip_ready_payload set')
+                continue
+            if entity.__class__.__name__.startswith("Activitypub") or entity.__class__.__name__.startswith("Diaspora"):
+                # Don't try to do anything with these entities currently
+                skip_ready_payload["matrix"] = True
+                logger.debug('Skipping matrix payload as payload is activitypub or diaspora')
+                continue
+            payload_info = []
+            # noinspection PyBroadException
+            try:
+                try:
+                    # For matrix we actually might get multiple payloads and endpoints
+                    payload_info = handle_create_payload(
+                        entity, author_user, protocol, parent_user=parent_user, payload_logger=payload_logger,
+                    )
+                except ValueError as ex:
+                    # No point continuing for this protocol
+                    skip_ready_payload["matrix"] = True
+                    logger.warning("handle_send - skipping matrix due to failure to generate payload: %s", ex)
+                    continue
+                if not matrix_config:
+                    matrix_config = get_matrix_configuration()
+                for payload in payload_info:
+                    rendered_payload = json.dumps(payload["payload"]).encode("utf-8")
+                    payloads.append({
+                        "auth": None,
+                        "headers": {
+                            "Authorization": f"Bearer {matrix_config['appservice']['token']}",
+                            "Content-Type": "application/json",
+                        },
+                        "payload": rendered_payload,
+                        "urls": {payload["endpoint"]},
+                        "method": payload.get("method"),
+                    })
+            except Exception:
+                logger.error(
+                    "handle_send - failed to generate matrix payload for %s, %s: %s",
+                    fid, endpoint, traceback.format_exc(),
+                    extra={
+                        "recipient": recipient,
+                        "unique_recipients": list(unique_recipients),
+                        "payload_info": payload_info,
+                        "payloads": payloads,
+                        "ready_payloads": ready_payloads,
+                        "entity": entity,
+                        "author": author_user.id,
+                        "parent_user": parent_user.id if parent_user else None,
+                    }
+                )
+                logger.debug('Continuing from matrix payload after error')
+                continue
 
     # Add public diaspora payload
     if ready_payloads["diaspora"]["payload"]:
         payloads.append({
-            "urls": ready_payloads["diaspora"]["urls"], "payload": ready_payloads["diaspora"]["payload"],
-            "content_type": "application/magic-envelope+xml", "auth": None,
+            "auth": None,
+            "headers": {
+                "Content-Type": "application/magic-envelope+xml",
+            },
+            "payload": ready_payloads["diaspora"]["payload"],
+            "urls": ready_payloads["diaspora"]["urls"],
         })
 
     logger.debug("handle_send - %s", payloads)
@@ -265,11 +358,13 @@ def handle_send(
     for payload in payloads:
         for url in payload["urls"]:
             try:
+                # TODO send_document and fetch_document need to handle rate limits
                 send_document(
                     url,
                     payload["payload"],
-                    auth=payload["auth"],
-                    headers={"Content-Type": payload["content_type"]},
+                    auth=payload.get("auth"),
+                    headers=payload.get("headers"),
+                    method=payload.get("method"),
                 )
             except Exception as ex:
                 logger.error("handle_send - failed to send payload to %s: %s, payload: %s", url, ex, payload["payload"])
