@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 from unittest.mock import patch
 
 import pytest
@@ -6,12 +5,13 @@ import pytest
 from federation.exceptions import NoSuitableProtocolFoundError
 from federation.inbound import handle_receive
 from federation.protocols.diaspora.protocol import Protocol
-from federation.tests.fixtures.payloads import UNENCRYPTED_DIASPORA_PAYLOAD
+from federation.tests.fixtures.payloads import DIASPORA_PUBLIC_PAYLOAD
+from federation.types import RequestType
 
 
-class TestHandleReceiveProtocolIdentification(object):
+class TestHandleReceiveProtocolIdentification:
     def test_handle_receive_routes_to_identified_protocol(self):
-        payload = UNENCRYPTED_DIASPORA_PAYLOAD
+        payload = RequestType(body=DIASPORA_PUBLIC_PAYLOAD)
         with patch.object(
                     Protocol,
                     'receive',
@@ -23,6 +23,6 @@ class TestHandleReceiveProtocolIdentification(object):
             assert mock_receive.called
 
     def test_handle_receive_raises_on_unidentified_protocol(self):
-        payload = "foobar"
+        payload = RequestType(body="foobar")
         with pytest.raises(NoSuitableProtocolFoundError):
             handle_receive(payload)
