@@ -3,11 +3,11 @@ import datetime
 import logging
 import re
 import socket
-from typing import Optional, Tuple
+from typing import Optional, Dict
 from urllib.parse import quote
+from uuid import uuid4
 
 import requests
-from ipdata import ipdata
 from requests.exceptions import RequestException, HTTPError, SSLError
 from requests.exceptions import ConnectionError
 from requests.structures import CaseInsensitiveDict
@@ -31,26 +31,6 @@ def fetch_content_type(url: str) -> Optional[str]:
         return response.headers.get('Content-Type')
 
 
-def fetch_country_by_ip(ip):
-    """
-    Fetches country code by IP
-
-    Returns empty string if the request fails in non-200 code.
-
-    Uses the ipdata.co service which has the following rules:
-
-    * Max 1500 requests per day
-
-    See: https://ipdata.co/docs.html#python-library
-    """
-    iplookup = ipdata.IPData()
-    data = iplookup.lookup(ip)
-    if data.get('status') != 200:
-        return ''
-
-    return data.get('response', {}).get('country_code', '')
-
-
 def fetch_document(url=None, host=None, path="/", timeout=10, raise_ssl_errors=True, extra_headers=None):
     """Helper method to fetch remote document.
 
@@ -128,17 +108,20 @@ def fetch_host_ip(host: str) -> str:
     return ip
 
 
-def fetch_host_ip_and_country(host: str) -> Tuple:
+def fetch_file(url: str, timeout: int = 30, extra_headers: Dict = None) -> str:
     """
-    Fetch ip and country by host
+    Download a file with a temporary name and return the name.
     """
-    ip = fetch_host_ip(host)
-    if not host:
-        return '', ''
-
-    country = fetch_country_by_ip(ip)
-
-    return ip, country
+    headers = {'user-agent': USER_AGENT}
+    if extra_headers:
+        headers.update(extra_headers)
+    response = requests.get(url, timeout=timeout, headers=headers, stream=True)
+    response.raise_for_status()
+    name = f"/tmp/{str(uuid4())}"
+    with open(name, "wb") as f:
+        for chunk in response.iter_content(chunk_size=8192):
+            f.write(chunk)
+    return name
 
 
 def parse_http_date(date):
@@ -191,7 +174,7 @@ def parse_http_date(date):
         raise ValueError("%r is not a valid date" % date) from exc
 
 
-def send_document(url, data, timeout=10, *args, **kwargs):
+def send_document(url, data, timeout=10, method="post", *args, **kwargs):
     """Helper method to send a document via POST.
 
     Additional ``*args`` and ``**kwargs`` will be passed on to ``requests.post``.
@@ -199,9 +182,12 @@ def send_document(url, data, timeout=10, *args, **kwargs):
     :arg url: Full url to send to, including protocol
     :arg data: Dictionary (will be form-encoded), bytes, or file-like object to send in the body
     :arg timeout: Seconds to wait for response (defaults to 10)
+    :arg method: Method to use, defaults to post
     :returns: Tuple of status code (int or None) and error (exception class instance or None)
     """
-    logger.debug("send_document: url=%s, data=%s, timeout=%s", url, data, timeout)
+    logger.debug("send_document: url=%s, data=%s, timeout=%s, method=%s", url, data, timeout, method)
+    if not method:
+        method = "post"
     headers = CaseInsensitiveDict({
         'User-Agent': USER_AGENT,
     })
@@ -211,10 +197,12 @@ def send_document(url, data, timeout=10, *args, **kwargs):
     kwargs.update({
         "data": data, "timeout": timeout, "headers": headers
     })
+    request_func = getattr(requests, method)
     try:
-        response = requests.post(url, *args, **kwargs)
+        response = request_func(url, *args, **kwargs)
         logger.debug("send_document: response status code %s", response.status_code)
         return response.status_code, None
+    # TODO support rate limit 429 code
     except RequestException as ex:
         logger.debug("send_document: exception %s", ex)
         return None, ex
